/* (C) 2022 YiRing, Inc. */
package com.yiring.app.domain.location;

import com.alibaba.fastjson.JSONObject;
import com.vladmihalcea.hibernate.type.json.JsonType;
import com.yiring.auth.domain.user.User;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import org.hibernate.annotations.Comment;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.Type;
import org.hibernate.annotations.TypeDef;
import org.locationtech.jts.geom.Point;

/**
 * 定位数据
 * 引用: 定位平台接口规范V3.0.1 #6
 * TODO:
 *  1. 实时数据日志的存储结构 TimeScale: 时间粒度，每天一个分区
 *      eg: SELECT create_hypertable('BS_LOCATION_LOG', 'time', chunk_time_interval => INTERVAL '1 day')
 *
 * @author Jim
 * @version 0.1
 * 2022/4/7 10:21
 */

@Getter
@Setter
@ToString
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@TypeDef(name = "json", typeClass = JsonType.class)
@Table(name = "BS_LOCATION_LOG")
@Comment("定位数据")
public class LocationLog implements Serializable {

    @Serial
    private static final long serialVersionUID = 3467455881020691989L;

    @Id
    @Comment("时间")
    @Column(nullable = false, columnDefinition = "timestamp without time zone")
    @CreationTimestamp
    LocalDateTime time;

    @Comment("地图 ID")
    Long areaId;

    @Comment("楼层")
    String floor;

    /**
     * 来源于定位数据产生时刻标签所属的人员
     */
    @Comment("用户")
    @ManyToOne
    @JoinColumn(name = "user_id")
    User user;

    /**
     * 来源于定位标签定位时刻的人员状态
     */
    @Comment("用户状态")
    String status;

    @Comment("标签")
    @ManyToOne
    @JoinColumn(name = "tag_id")
    LocationTag tag;

    @Comment("坐标点信息")
    @Type(type = "jts_geometry")
    @Column(columnDefinition = "point")
    Point point;

    @Comment("信标集合")
    @Builder.Default
    @OneToMany
    @ToString.Exclude
    Set<LocationBeacon> beacons = new HashSet<>(0);

    @Comment("围栏集合")
    @Builder.Default
    @OneToMany
    @ToString.Exclude
    Set<LocationFence> fences = new HashSet<>(0);

    @Comment("静止/运动")
    Boolean silent;

    @Comment("电量")
    Integer volt;

    @Comment("电量单位")
    String voltUnit;

    @Comment("原始数据")
    @org.hibernate.annotations.Type(type = "json")
    @Column(columnDefinition = "json")
    JSONObject raw;

    @Comment("定位时间")
    LocalDateTime locationTime;
}
