/* (C) 2022 YiRing, Inc. */
package com.yiring.app.domain.log;

import com.alibaba.fastjson.JSONObject;
import com.vladmihalcea.hibernate.type.json.JsonBinaryType;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import org.hibernate.annotations.Comment;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.TypeDef;

/**
 * 真源定位系统实时数据日志
 * TODO:
 *  1. 实时数据日志的存储结构 TimeScale: 时间粒度，每小时一个分区
 *      eg: SELECT create_hypertable('ZY_REALTIME_LOG', 'time', chunk_time_interval => INTERVAL '1 hour')
 *  2. 数据分区压缩
 *  3. 定时删除过期分区数据
 *  参考：<a href="https://blog.csdn.net/yang_z_1/article/details/111560747">文档</a>
 *
 * @author Jim
 * @version 0.1
 * 2022/4/25 15:52
 */

@Getter
@Setter
@ToString
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@TypeDef(name = "jsonb", typeClass = JsonBinaryType.class)
@Table(name = "ZY_REALTIME_LOG", indexes = { @Index(columnList = "method") })
@Comment("真源定位系统实时数据日志")
public class ZyRealtimeLog implements Serializable {

    @Serial
    private static final long serialVersionUID = 5545864821082386L;

    @Id
    @Comment("时间")
    @Column(nullable = false, columnDefinition = "timestamp without time zone")
    @CreationTimestamp
    LocalDateTime time;

    @Comment("类型")
    @Column(nullable = false)
    String method;

    @Comment("内容")
    @org.hibernate.annotations.Type(type = "jsonb")
    @Column(nullable = false, columnDefinition = "jsonb")
    JSONObject raw;
}
