/* (C) 2022 YiRing, Inc. */
package com.yiring.app.rabbit.config;

import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AcknowledgeMode;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.listener.RabbitListenerContainerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.amqp.SimpleRabbitListenerContainerFactoryConfigurer;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 真源 RabbitMQ 订阅配置
 *
 * @author Jim
 * @version 0.1
 * 2022/4/13 17:05
 */

@Slf4j
@Configuration
@ConditionalOnProperty(prefix = "zy-config.rabbitmq", value = "enabled", havingValue = "true")
public class ZyRabbitConfig {

    @Resource
    ZyConfigProperties.ZyConfigRabbitmq rabbitmq;

    @Resource
    ConfigurableApplicationContext context;

    public static final String CONNECTION_FACTORY_NAME = "zyRabbitConnectionFactory";
    public static final String LISTENER_FACTORY_NAME = "zyRabbitListenerFactory";

    /**
     * 消息队列名称（必须要与配置文件中的 queue-name 完全一致）
     * 规则: tenant_msg_${open.client-secret}_${open.client-id}
     * 参见文档: 定位平台接口规范V3.0.1 - 通用版.pdf #6
     */
    public static final String MESSAGE_QUEUES_NAME = "tenant_msg_12A14FDC_sc21080400";

    @Bean(CONNECTION_FACTORY_NAME)
    public ConnectionFactory zyConnectionFactory() {
        return RabbitConfig.connectionFactory(
            rabbitmq.getHost(),
            rabbitmq.getPort(),
            rabbitmq.getUsername(),
            rabbitmq.getPassword(),
            rabbitmq.getVirtualHost()
        );
    }

    @Bean(LISTENER_FACTORY_NAME)
    public RabbitListenerContainerFactory<?> rabbitListenerContainerFactory(
        SimpleRabbitListenerContainerFactoryConfigurer configurer,
        @Qualifier(CONNECTION_FACTORY_NAME) ConnectionFactory connectionFactory
    ) {
        // 检查队列名称是否与配置文件一致，避免监听错误
        if (!MESSAGE_QUEUES_NAME.equals(rabbitmq.getQueueName())) {
            log.error("队列名称不一致，请检查配置文件");
            context.close();
            return null;
        }

        SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
        // 手动确认消息模式
        factory.setAcknowledgeMode(AcknowledgeMode.MANUAL);
        configurer.configure(factory, connectionFactory);
        return factory;
    }
}
