/* (C) 2022 YiRing, Inc. */
package com.yiring.app.util;

import lombok.experimental.UtilityClass;
import org.locationtech.jts.geom.Coordinate;
import org.locationtech.jts.geom.GeometryFactory;
import org.locationtech.jts.geom.Point;

/**
 * 空间函数工具类
 *
 * @author Jim
 * @version 0.1
 * 2022/4/15 9:52
 */

@SuppressWarnings({ "unused" })
@UtilityClass
public class GeoUtils {

    public final GeometryFactory factory = new GeometryFactory();

    /**
     * 创建点
     *
     * @param lon 经度
     * @param lat 纬度
     * @return 点
     */
    public Point createPoint(double lon, double lat) {
        return factory.createPoint(new Coordinate(lon, lat));
    }

    /**
     * 创建一个三维点
     * @param lon 经度
     * @param lat 纬度
     * @param alt 高度
     * @return 点
     */
    public Point createPoint(double lon, double lat, double alt) {
        return factory.createPoint(new Coordinate(lon, lat, alt));
    }

    /**
     * 相对坐标转换为经纬度
     * @param x x 坐标值
     * @param y y 坐标值
     * @param z z 坐标值
     * @return 经纬度点
     */
    public Point xyzToPoint(double x, double y, double z) {
        // TODO
        // 根据真源定位系统设置的坐标系，转换为经纬度
        // 根据北向的地图左下角坐标点，矩形长宽距离，结合二维图片的像素比，计算经纬度
        return factory.createPoint(new Coordinate(x, y, z));
    }
}
