/* (C) 2021 YiRing, Inc. */
package com.yiring.app.web;

import com.yiring.app.constant.Code;
import com.yiring.app.exception.CodeException;
import com.yiring.app.util.zy.ZyUtil;
import com.yiring.common.core.Result;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import java.util.Arrays;
import java.util.List;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Slf4j
@Validated
@Api(tags = "Hello")
@RequestMapping("/hello/")
@RestController
public class HelloController {

    String text = "😎 Hello World";

    @GetMapping
    public Result<String> hello() {
        return Result.ok(text);
    }

    /**
     * 测试失败自定义状态信息输出
     */
    @GetMapping("fail")
    public Result<String> fail() {
        throw new CodeException(Code.FAIL);
    }

    @GetMapping("page")
    public Result<PageVo<String>> page(@Valid PageParam pageParam) {
        log.info("PageParam: {}", pageParam);

        List<String> data = Arrays.asList(text.split(" "));
        PageVo<String> vo = PageVo.build(data, data.size());
        return Result.ok(vo);
    }

    @GetMapping("test")
    public Result<String> test() {
        return Result.ok(ZyUtil.manageLogin());
    }
}
